#!/usr/bin/env python3

import threading
import time
import random
import sys
import os
import json
from datetime import datetime

current_dir = os.path.dirname(os.path.realpath(__file__))
parent_dir = os.path.dirname(current_dir)
sys.path.append(parent_dir)

from lib.sys_utils import IqLogger


class CachedNetworkStatus:
    _instance = None
    _lock = threading.RLock()
    NETWORK_STATUS_CACHE_PATH = '/tmp/network_status_cache.json'  # Путь к файлу кэша статуса сети

    def __new__(cls):
        if cls._instance is None:
            with cls._lock:
                if cls._instance is None:
                    cls._instance = super(CachedNetworkStatus, cls).__new__(cls)
                    cls._instance._initialize_data()
        return cls._instance

    def _initialize_data(self):
        self._data = {
            "Connections": {
                "Cellular": {
                    "ModemInfo": {},
                    "SIMCards": {}
                },
                "WiFi": {},
                "Ethernet": {}
            },
            "Priorities": {},
            "Datetime": "",
            "EventTimestamp": 0,
            "Version": "1.0"
        }

    def update_time(self):
        current_time = datetime.now()
        self._data["Datetime"] = current_time.strftime("%Y-%m-%d %H:%M:%S.%f")
        self._data["EventTimestamp"] = time.mktime(current_time.timetuple()) + current_time.microsecond / 1e6

    @property
    def lock(self):
        return self._lock

    @property
    def data(self):
        return self._data

    def save(self):
        try:
            with self.lock:
                with open(self.NETWORK_STATUS_CACHE_PATH, 'w') as file:
                    json.dump(self._data, file)
            IqLogger().info(f"Data successfully saved to {self.NETWORK_STATUS_CACHE_PATH}")
        except Exception as e:
            IqLogger().error(f"Failed to save data to {self.NETWORK_STATUS_CACHE_PATH}: {e}")

    def restore(self):
        try:
            with self.lock:
                if not os.path.exists(self.NETWORK_STATUS_CACHE_PATH):
                    raise FileNotFoundError(f"No data file found at {self.NETWORK_STATUS_CACHE_PATH}")

                with open(self.NETWORK_STATUS_CACHE_PATH, 'r') as file:
                    self._data = json.load(file)
            IqLogger().info(f"Data successfully restored from {self.NETWORK_STATUS_CACHE_PATH}")
            return True
        except FileNotFoundError as e:
            IqLogger().warning(f"{e}")
        except json.JSONDecodeError as e:
            IqLogger().error(f"Failed to decode JSON from {self.NETWORK_STATUS_CACHE_PATH}: {e}")
        except Exception as e:
            IqLogger().error(f"Failed to restore data from {self.NETWORK_STATUS_CACHE_PATH}: {e}")
        return False

def safe_threading_test(thread_num, iterations):
    cached_data = CachedNetworkStatus()

    for i in range(iterations):
        # Случайная задержка перед блокировкой
        time.sleep(random.uniform(0.01, 0.1))

        # Первый захват RLock
        with cached_data.lock:
            # Обновляем время (просто пример работы с общим объектом)
            cached_data.update_time()

            # Второй (вложенный) захват той же RLock
            with cached_data.lock:
                # Инициализируем счётчик, если его нет
                if "Counter" not in cached_data.data:
                    cached_data.data["Counter"] = 0

                cached_data.data["Counter"] += 1

                print(
                    f"Thread {thread_num} iteration {i}: "
                    f"updated 'Counter' to {cached_data.data['Counter']}, "
                    f"time: {cached_data.data['Datetime']}"
                )

            # После вложенного `with` вызываем `save()` — внутри тоже используется self.lock
            cached_data.save()

        # Случайная задержка после разблокировки
        time.sleep(random.uniform(0.001, 0.01))

# Запуск тестов, если скрипт запущен с аргументом 'test'
if __name__ == "__main__":
    if len(sys.argv) > 1 and sys.argv[1] == "test":
        threads = []
        for i in range(300):
            thread = threading.Thread(target=safe_threading_test, args=(i, 100))
            threads.append(thread)

        for thread in threads:
            thread.start()

        for thread in threads:
            thread.join()
