#!/usr/bin/env python3

import re
import sys
import json
import argparse
import logging
from datetime import datetime
from collections import defaultdict

can_plot = True
try:
    import matplotlib.pyplot as plt
    import matplotlib.dates as mdates
    from matplotlib.ticker import MaxNLocator
except ImportError:
    can_plot = False

can_tabulate = True
try:
    from tabulate import tabulate
except ImportError:
    can_tabulate = False

state_times = {
    "wb-gsm-sim1": defaultdict(lambda: 0),
    "wb-gsm-sim2": defaultdict(lambda: 0)
}
state_counts = {
    "wb-gsm-sim1": defaultdict(lambda: 0),
    "wb-gsm-sim2": defaultdict(lambda: 0)
}
current_state = {}
current_time = {}
current_sim = None

sim_intervals = {
    "wb-gsm-sim1": [],
    "wb-gsm-sim2": []
}

slot_current_imsi = {
    "wb-gsm-sim1": None,
    "wb-gsm-sim2": None
}

slot_sessions = {
    "wb-gsm-sim1": {},
    "wb-gsm-sim2": {}
}

current_operator = {
    "wb-gsm-sim1": None,
    "wb-gsm-sim2": None
}
current_imsi_data = {
    "wb-gsm-sim1": None,
    "wb-gsm-sim2": None
}
current_iccid = {
    "wb-gsm-sim1": None,
    "wb-gsm-sim2": None
}
current_access_tech = {
    "wb-gsm-sim1": None,
    "wb-gsm-sim2": None
}
current_signal_quality = {
    "wb-gsm-sim1": None,
    "wb-gsm-sim2": None
}

skip_next_info = {
    "wb-gsm-sim1": False,
    "wb-gsm-sim2": False
}

last_signal_timestamp = {
    "wb-gsm-sim1": defaultdict(lambda: None),
    "wb-gsm-sim2": defaultdict(lambda: None)
}

last_line_time = None

def configure_logging(level, quiet):
    if quiet:
        level = logging.ERROR
    logging.basicConfig(level=level, format='%(asctime)s - %(levelname)s - %(message)s')

def parse_arguments():
    parser = argparse.ArgumentParser(description="Analyze SIM logs and optionally output table, JSON, Gantt chart and ratings.")
    parser.add_argument("logfile", nargs="?", help="Path to log file. If omitted, read from stdin.")
    parser.add_argument("--no-graph", action="store_true", help="Do not generate and display any graphs.")
    parser.add_argument("--json", action="store_true", help="Output data in JSON format.")
    parser.add_argument("--no-table", action="store_true", help="Do not output summary table.")
    parser.add_argument("--rating-json", action="store_true", help="Output rating data in JSON format only and exit.")
    parser.add_argument("--quiet", action="store_true", help="Suppress most log messages (only errors are shown).")
    parser.add_argument("--log-level", default="INFO", choices=["DEBUG", "INFO", "WARNING", "ERROR", "CRITICAL"],
                        help="Set the logging level (default: INFO).")
    return parser.parse_args()

def read_log_file(args):
    if not args.logfile:
        if sys.stdin.isatty():
            logging.error("No input file provided and no data piped. Exiting.")
            sys.exit(1)
        return sys.stdin
    else:
        try:
            return open(args.logfile, 'r')
        except FileNotFoundError:
            logging.error(f"File not found: {args.logfile}")
            sys.exit(1)

from datetime import datetime

last_datetime = None
current_year = datetime.now().year

# В журнале не упоминается год, поэтому предполагаем что год текущий.
# Если произошла смена года, значит год был точно не текущий, но делать нечего
# прибавляем один год к новым данным.
# Такой логики достаточно для анализа логов за день
# и поиска актуального уровня сигнала.
def parse_log_time(line):
    global last_datetime, current_year
    match = re.match(r'^(\w{3} \d{2} \d{2}:\d{2}:\d{2})', line)
    if match:
        base_dt = datetime.strptime(match.group(1), "%b %d %H:%M:%S")
        # Предположим начальный год
        if last_datetime is None:
            dt = base_dt.replace(year=current_year)
            last_datetime = dt
            return dt
        else:
            # Присваиваем текущий год
            dt = base_dt.replace(year=current_year)
            # Если новая дата меньше предыдущей, значит произошло переполнение года.
            # На всякий случай проверяем что сдвиг значимый (от 30-ти дней).
            if dt < last_datetime and (last_datetime - dt).days >= 30:
                current_year += 1
                dt = base_dt.replace(year=current_year)
            last_datetime = dt
            return dt
    return None

def reset_sim_data(sim):
    current_operator[sim] = None
    current_imsi_data[sim] = None
    current_iccid[sim] = None
    current_access_tech[sim] = None
    current_signal_quality[sim] = None
    slot_current_imsi[sim] = None

def record_signal(sim, time_stamp):
    if sim and slot_current_imsi[sim] and current_signal_quality[sim] is not None \
       and current_access_tech[sim] is not None and current_operator[sim] is not None:
        imsi = slot_current_imsi[sim]
        if imsi not in slot_sessions[sim]:
            slot_sessions[sim][imsi] = []
        slot_sessions[sim][imsi].append((time_stamp,
                                         current_signal_quality[sim],
                                         current_access_tech[sim],
                                         current_operator[sim]))
        last_signal_timestamp[sim][imsi] = time_stamp

def process_log(input_file):
    global current_sim, last_line_time
    logging.debug("Reading log lines...")
    for line in input_file:
        time_stamp = parse_log_time(line)
        if not time_stamp:
            continue
        last_line_time = time_stamp

        if "Activation: starting connection 'wb-gsm-sim" in line:
            new_sim = None
            if "wb-gsm-sim1" in line:
                new_sim = "wb-gsm-sim1"
            elif "wb-gsm-sim2" in line:
                new_sim = "wb-gsm-sim2"

            if new_sim:
                if current_sim and current_sim != new_sim:
                    reset_sim_data(current_sim)
                current_sim = new_sim
                current_state[current_sim] = "activating"
                current_time[current_sim] = time_stamp
                skip_next_info[current_sim] = True
            continue

        if not current_sim:
            continue

        state_match = re.search(r'3GPP registration state changed \((.*?) -> (.*?)\)', line)
        if state_match:
            prev_state, new_state = state_match.groups()
            if current_sim in current_state and current_state[current_sim] == prev_state:
                elapsed = (time_stamp - current_time[current_sim]).total_seconds()
                state_times[current_sim][prev_state] += elapsed
                sim_intervals[current_sim].append((current_time[current_sim], time_stamp, prev_state))

            state_counts[current_sim][new_state] += 1
            current_state[current_sim] = new_state
            current_time[current_sim] = time_stamp

        tmp_operator = None
        tmp_imsi = None
        tmp_iccid = None
        tmp_signal_quality = None
        tmp_access_tech = None

        op_match = re.search(r'operator name:\s+(.*)', line)
        if op_match:
            tmp_operator = op_match.group(1).strip()

        imsi_match = re.search(r'imsi:\s+(\d+)', line)
        if imsi_match:
            tmp_imsi = imsi_match.group(1)

        iccid_match = re.search(r'iccid:\s+(\d+)', line)
        if iccid_match:
            tmp_iccid = iccid_match.group(1)

        signal_match = re.search(r'signal quality:\s+(\d+)%', line)
        if signal_match:
            tmp_signal_quality = int(signal_match.group(1))

        access_match = re.search(r'access tech:\s+(\S+)', line)
        if access_match:
            tmp_access_tech = access_match.group(1).strip()

        if skip_next_info[current_sim]:
            if (tmp_operator or tmp_imsi or tmp_iccid or
                tmp_signal_quality is not None or tmp_access_tech is not None):
                skip_next_info[current_sim] = False
            continue

        if tmp_operator is not None:
            current_operator[current_sim] = tmp_operator
        if tmp_imsi is not None:
            current_imsi_data[current_sim] = tmp_imsi
        if tmp_iccid is not None:
            current_iccid[current_sim] = tmp_iccid
        if tmp_signal_quality is not None:
            current_signal_quality[current_sim] = tmp_signal_quality
        if tmp_access_tech is not None:
            current_access_tech[current_sim] = tmp_access_tech

        if current_imsi_data[current_sim] and current_imsi_data[current_sim] != slot_current_imsi[current_sim]:
            slot_current_imsi[current_sim] = current_imsi_data[current_sim]

        if (slot_current_imsi[current_sim] and
            current_operator[current_sim] and
            current_access_tech[current_sim] is not None and
            current_signal_quality[current_sim] is not None):
            record_signal(current_sim, time_stamp)

    logging.debug("Finished reading lines.")
    if last_line_time:
        for sim in ["wb-gsm-sim1", "wb-gsm-sim2"]:
            if sim in current_state and sim in current_time:
                last_state = current_state[sim]
                start_time = current_time[sim]
                if start_time < last_line_time:
                    elapsed = (last_line_time - start_time).total_seconds()
                    state_times[sim][last_state] += elapsed
                    sim_intervals[sim].append((start_time, last_line_time, last_state))

def format_summary(times, counts):
    state_order = ["unknown", "registering", "home", "roaming", "idle", "denied", "activating"]
    headers = ["State", "SIM1 Time (s)", "SIM2 Time (s)", "SIM1 Count", "SIM2 Count"]
    states = set(times["wb-gsm-sim1"].keys()).union(times["wb-gsm-sim2"].keys())
    sorted_states = sorted(states, key=lambda s: state_order.index(s) if s in state_order else len(state_order))

    rows = []
    for state in sorted_states:
        rows.append([
            state,
            int(times["wb-gsm-sim1"].get(state, 0)),
            int(times["wb-gsm-sim2"].get(state, 0)),
            counts["wb-gsm-sim1"].get(state, 0),
            counts["wb-gsm-sim2"].get(state, 0)
        ])
    return headers, rows

def calculate_summary():
    return format_summary(state_times, state_counts)

def print_table(headers, rows):
    if not can_tabulate:
        logging.warning("ModuleNotFoundError: No module named \'tabulate\'")
    print(tabulate(rows, headers, tablefmt="pretty"))

def print_json_data(times, counts):
    data = {
        "sim1": {
            "states": dict(times["wb-gsm-sim1"]),
            "counts": dict(counts["wb-gsm-sim1"])
        },
        "sim2": {
            "states": dict(times["wb-gsm-sim2"]),
            "counts": dict(counts["wb-gsm-sim2"])
        }
    }
    print(json.dumps(data, indent=4))


def calculate_ratings(slot_sessions):
    tech_weights = {
        "gsm": 0.1,
        "edge": 0.2,
        "umts": 0.25,
        "hspa": 0.5,
        "lte": 1
    }

    ratings = {}
    for sim, imsi_data in slot_sessions.items():
        ratings[sim] = {}
        for imsi, records in imsi_data.items():
            if not records or len(records) < 10:
                ratings[sim][imsi] = {
                    "avg_signal_rating": 0,
                    "records": len(records),
                    "last_timestamp": last_signal_timestamp[sim].get(imsi)
                }
                continue

            weighted_signal_sum = 0
            count = 0

            for (timestamp, signal_quality, tech, operator) in records:
                tech_lower = tech.lower()
                if tech_lower not in tech_weights:
                    logging.warning(f"Неизвестная технология доступа: {tech}")
                    continue

                weight = tech_weights[tech_lower]
                weighted_signal_sum += signal_quality * weight
                count += 1

            avg_rating = weighted_signal_sum / count if count > 0 else 0

            ratings[sim][imsi] = {
                "avg_signal_rating": avg_rating,
                "records": len(records),
                "last_timestamp": last_signal_timestamp[sim].get(imsi)
            }

    return ratings

def print_rating_json(ratings):
    logging.info("print_rating_json")
    def json_serial(obj):
        if isinstance(obj, datetime):
            return {
                "iso": obj.isoformat(),         # Время в ISO-формате
                "timestamp": int(obj.timestamp())  # Время в timestamp
            }
        raise TypeError(f"Type {type(obj)} not serializable")

    print(json.dumps(ratings, default=json_serial, indent=4))

def print_average_ratings(ratings):
    logging.info("\nAverage Signal Ratings per SIM:")
    for sim, imsi_ratings in ratings.items():
        if not imsi_ratings:
            avg = 0
        else:
            total_weighted_rating = sum(r['avg_signal_rating'] * r['records'] for r in imsi_ratings.values())
            total_records = sum(r['records'] for r in imsi_ratings.values())
            avg = total_weighted_rating / total_records if total_records > 0 else 0
        logging.info(f"{sim}: {avg:.2f}")

def plot_results(no_graph, slot_sessions, sim_intervals):
    if no_graph or not can_plot:
        return

    fig, (ax1, ax2) = plt.subplots(2, 1, figsize=(14, 10), constrained_layout=True, sharex=True)

    state_colors = {
        "unknown": "white",
        "registering": "yellow",
        "home": "green",
        "roaming": "blue",
        "idle": "orange",
        "denied": "red",
        "activating": "purple"
    }

    ax1.xaxis.set_major_formatter(mdates.DateFormatter('%H:%M:%S'))
    ax2.tick_params(axis='x', labelrotation=15)
    ax1.xaxis.set_major_locator(MaxNLocator(16))

    ax1.grid(True, axis='x', linestyle='--', linewidth=0.5)

    y_height = 1
    y_positions = {
        "wb-gsm-sim1": 1,
        "wb-gsm-sim2": 0
    }

    for sim, intervals in sim_intervals.items():
        for start, end, state in intervals:
            start_num = mdates.date2num(start)
            end_num = mdates.date2num(end)
            duration = end_num - start_num
            ax1.barh(y_positions[sim], duration, left=start_num, height=y_height,
                     color=state_colors.get(state, "black"), edgecolor='none')

    ax1.set_yticks([y_positions["wb-gsm-sim1"], y_positions["wb-gsm-sim2"]])
    ax1.set_yticklabels(["SIM1", "SIM2"])
    ax1.set_xlabel("Time")
    ax1.set_ylabel("SIM")
    ax1.set_title("SIM Registration States")

    legend_elements = [plt.Line2D([0], [0], color=c, lw=4, label=s) for s, c in state_colors.items()]
    ax1.legend(handles=legend_elements, title="Registration States", bbox_to_anchor=(1.02, 1), loc='upper left')

    tech_colors = {
        "gsm": "gray",
        "edge": "red",
        "umts": "orange",
        "hspa": "blue",
        "lte": "green"
    }

    sim_markers = {
        "wb-gsm-sim1": "x",
        "wb-gsm-sim2": "D",
    }

    all_times = []
    for sim, imsi_data in slot_sessions.items():
        marker = sim_markers.get(sim, 'o')
        for imsi, records in imsi_data.items():
            if not records:
                continue
            times = [r[0] for r in records]
            qualities = [r[1] for r in records]
            techs = [r[2] for r in records]
            ops = [r[3] for r in records]

            if times:
                all_times.extend(times)
                op_name = ops[-1] if ops else "Unknown"
                last_tech = techs[-1] if techs else "unknown"
                color = tech_colors.get(last_tech.lower(), "black")
                label = f"{sim} | IMSI: {imsi} | {op_name}"
                ax2.scatter(times, qualities, marker=marker, color=color, label=label, alpha=0.5, s=20)

    if all_times:
        ax2.set_xlim(min(all_times), max(all_times))

    ax2.xaxis.set_major_formatter(mdates.DateFormatter('%H:%M:%S %d/%m'))
    ax2.xaxis.set_major_locator(MaxNLocator(16))
    ax2.set_xlabel("Time")
    ax2.set_ylabel("Signal Quality (%)")
    ax2.set_title("Signal Quality")

    ax2.grid(True, axis='x', linestyle='--', linewidth=0.5)

    import matplotlib.lines as mlines
    tech_handles = [mlines.Line2D([], [], color=c, marker='o', linestyle='none', label=t.upper()) for t, c in tech_colors.items()]
    sim_handles = [mlines.Line2D([], [], color='black', marker=m, linestyle='none', label=s) for s, m in sim_markers.items()]

    legend_tech = ax2.legend(handles=tech_handles, title="Access Technology", loc='upper left', bbox_to_anchor=(1.02, 1))
    ax2.add_artist(legend_tech)
    legend_sim = ax2.legend(handles=sim_handles, title="SIM", loc='upper left', bbox_to_anchor=(1.02, 0.6))
    plt.gcf().canvas.manager.set_window_title("SIM Analysis: Signal Quality and States")

    plt.show()

def main():
    args = parse_arguments()
    log_level = getattr(logging, args.log_level.upper(), logging.INFO)
    configure_logging(log_level, args.quiet)

    input_file = read_log_file(args)
    process_log(input_file)

    headers, rows = calculate_summary()

    ratings = calculate_ratings(slot_sessions)

    if args.rating_json:
        print_rating_json(ratings)
        return

    print_average_ratings(ratings)

    if args.json:
        print_json_data(state_times, state_counts)

    if not args.no_table and not args.json:
        print_table(headers, rows)

    plot_results(args.no_graph, slot_sessions, sim_intervals)

    if input_file is not sys.stdin:
        input_file.close()

    logging.debug("Done.")

if __name__ == "__main__":
    main()
