#!/usr/bin/env python3

import json
import os
import subprocess
import sys
import threading
import time
import re

current_dir = os.path.dirname(os.path.realpath(__file__))
parent_dir = os.path.dirname(current_dir)
sys.path.append(parent_dir)

from lib.sys_utils import IqLogger
from lib.mmcli2json import convert_mmcli_to_json
from lib.cached_network_status import CachedNetworkStatus
from lib.mqtt_client import MQTTClient

def execute_command(command, timeout=60):
    """
    Выполняет команду mmcli и возвращает результат выполнения.
    Выполняется с заданным таймаутом.
    """
    print(f"Выполнение: {command}")
    end_time = time.time() + timeout
    while time.time() < end_time:
        process = subprocess.Popen(command, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        output, error = process.communicate()

        if process.returncode == 0:
            print(f"Команда \"{command}\" выполнена. Результат:\n {output.decode()}")
            return output.decode()
        print(f"Ошибка при выполнении команды {command}: {error.decode()}")
        time.sleep(1)

    print(f"Не удалось выполнить команду {command} в течение {timeout} секунд")
    return None

def get_mmcli_json_data(mmcli_args, timeout=60):
    """
    Выполняет команду mmcli с заданными аргументами и возвращает данные в формате JSON.
    """
    command = f'mmcli {mmcli_args}'
    output = execute_command(command, timeout)
    if output is not None:
        return convert_mmcli_to_json(output)
    return None

def extract_sim_slots(sim_data):
    sim_slot_paths = sim_data.get("sim slot paths", {})
    result = {}

    for slot, path in sim_slot_paths.items():
        match = re.match(r"(.+?)\s*(\((.*?)\))?$", path)
        if match:
            path_only, _, status = match.groups()
            result[slot] = {
                "path": path_only,
                "status": status if status else None
            }

    return result

def process_sim_info(modem_info):
    sim_info = modem_info.get("SIM", {})
    slots = extract_sim_slots(sim_info)
    cached_data = CachedNetworkStatus()

    for slot, slot_info in slots.items():
        if slot_info["status"] == "active" and slot_info["path"] != "none":
            sim_card_info = get_mmcli_json_data(f'-i {slot_info["path"]}')
            if sim_card_info:
                tmp = sim_card_info.get("Properties", {})
                tmp.update(sim_card_info.get("General", {}))
                tmp["active"] = True

                with cached_data.lock:
                    cached_data.data["Connections"]["Cellular"]["SIMCards"].setdefault(slot, {})
                    cached_data.data["Connections"]["Cellular"]["SIMCards"][slot].update(tmp)
        else:
            with cached_data.lock:
                cached_data.data["Connections"]["Cellular"]["SIMCards"].setdefault(slot, {}).update({"active": False})

def update_modem_info():
    modem_info = get_mmcli_json_data('-m wbc', 60)
    if modem_info:
        process_sim_info(modem_info)

        cached_data = CachedNetworkStatus()
        with cached_data.lock:
            cached_data.update_time()
            cached_data.data["Connections"]["Cellular"]["ModemInfo"] = modem_info

def wait_for_network_registration_and_collect_info(timeout=300, interval=10):
    start_time = time.time()
    while time.time() - start_time < timeout:
        modem_info = get_mmcli_json_data('-m wbc', 60)
        if modem_info:
            status = modem_info.get("3GPP", {}).get("registration", "")
            print(f"Current state: {status}")
            if status.lower() == "home":
                print("SIM registered in the network.")
                for _ in range(15):
                    update_modem_info()
                    time.sleep(5)
                return True
        time.sleep(interval)
    print("Timeout: SIM not registered in the network.")
    return False

def collect_initial_sim_info():
    execute_command("service wb-connection-manager stop", timeout=60)
    start_time = time.time()
    time.sleep(5)

    modem_info = get_mmcli_json_data('-m wbc', 300)
    if modem_info:
        sim_info = modem_info.get("SIM", {})
        slots = extract_sim_slots(sim_info)
        cached_data = CachedNetworkStatus()

        for slot, slot_info in slots.items():

            sim_id = int(slot.split()[-1])
            execute_command(f"mmcli -m wbc --set-primary-sim-slot={sim_id}", timeout=60)
            time.sleep(10)

            sim_card_info = get_mmcli_json_data('-i any', 60)
            if sim_card_info:
                tmp = sim_card_info.get("Properties", {})
                tmp.update(sim_card_info.get("General", {}))
                with cached_data.lock:
                    cached_data.update_time()
                    cached_data.data["Connections"]["Cellular"]["SIMCards"][slot] = tmp

            wait_for_network_registration_and_collect_info()

    execute_command("mmcli -m wbc --set-primary-sim-slot=1", timeout=60)
    time.sleep(5)
    execute_command("service wb-connection-manager start", timeout=60)
    time.sleep(5)

    modem_info = get_mmcli_json_data('-m wbc', 60)
    if modem_info:
        process_sim_info(modem_info)

def modem_info_task(interval):
    while True:
        update_modem_info()
        time.sleep(interval)
