#!/usr/bin/env python3

import json
import os
import sys
import threading
import traceback
import paho.mqtt.client as mqtt
from .sys_utils import IQLog, except_log
from copy import deepcopy

current_dir = os.path.dirname(os.path.realpath(__file__))
parent_dir = os.path.dirname(current_dir)
sys.path.append(parent_dir)

from lib.sys_utils import IqLogger
from lib.sys_utils import ShellCommand
from . import settings as S

apply_settings_mutex = threading.Lock()

def restart_service_if_running(service_name):
    shell_command = ShellCommand(IqLogger())
    ret, out, _ = shell_command.command_exec(f"systemctl is-active {service_name}")

    if out.strip() == 'active':
        IqLogger().info(f"Service {service_name} is active. Restarting...")
        ret, out, err = shell_command.command_exec(f"systemctl restart {service_name}")

        if ret == 0:
            IqLogger().info(f"Service {service_name} has been restarted successfully.")
        else:
            IqLogger().error(f"Failed to restart service {service_name}. Error: {err}")
    else:
        IqLogger().info(f"Service {service_name} is not active. No action taken.")

def apply_settings():
    mqtt_client = MQTTClient(logger=IqLogger())
    with apply_settings_mutex:
        if mqtt_client.new_settings():
            settings = mqtt_client.get_settings()
            from lib.wifi_manager import manage_wifi_connection
            manage_wifi_connection(settings)
            restart_service_if_running('wb-connection-manager')

def interface_settings_are_changed(settings1, settings2):
    if (settings1 is not None) and (settings2 is not None):
        return settings1 != settings2
    if (settings1 is None) and (settings2 is None):
        return False
    return True

class MQTTClient(IQLog):
    _instance = None
    _lock = threading.Lock()

    def __new__(cls, *args, **kwargs):
        with cls._lock:
            if cls._instance is None:
                cls._instance = super(MQTTClient, cls).__new__(cls)
                cls._instance._initialized = False
            return cls._instance

    def __init__(self, logger=None):
        if self._initialized:
            return

        super().__init__(IqLogger())
        self._connected_flag = False
        self._settings_mutex = threading.Lock()
        self._settings = None
        self._new_settings_flag = False
        self._client = mqtt.Client()
        self._message_callbacks = [(S.SETTINGS_MQTT_TOPIC, self._on_settings)]
        self._client.on_connect = self._on_connect
        self._client.on_disconnect = self._on_disconnect

        self.start_mqtt_client()
        self._initialized = True

    def start_mqtt_client(self):
        if not hasattr(self, '_mqtt_thread') or not self._mqtt_thread.is_alive():
            self._mqtt_thread = threading.Thread(target=self._run_client)
            self._mqtt_thread.daemon = True
            self._mqtt_thread.start()

    def _run_client(self):
        self.logger.info("Connecting to MQTT")
        self._client.connect('localhost')
        self.logger.info("Connected to MQTT")
        self._client.loop_forever()
        self.logger.info("MQTT client loop finished")

    @except_log
    def _on_settings(self, client, userdata, msg):
        settings = self._process_json_payload(msg)
        self.logger.info("OLD SETTINGS: {0}".format(self._settings))
        self.logger.info("NEW SETTINGS: {0}".format(settings))
        with self._settings_mutex:
            if interface_settings_are_changed(self._settings, settings):
                self._settings = settings
                self._new_settings_flag = True
                self.logger.info("New settings applied")

                thrd = threading.Thread(target=apply_settings)
                thrd.daemon = True
                thrd.start()

    def new_settings(self):
        with self._settings_mutex:
            flag = self._new_settings_flag
            self._new_settings_flag = False
            return flag

    def get_settings(self):
        with self._settings_mutex:
            return deepcopy(self._settings) if self._settings is not None else None

    @except_log
    def _on_connect(self, client, userdata, flags, rc):
        txt = "Connection returned result: {0}".format(mqtt.connack_string(rc))
        if rc != mqtt.CONNACK_ACCEPTED:
            self._connected_flag = False
            self.logger.error(txt)
        else:
            self._connected_flag = True
            self.logger.info(txt)
            for topic, callback in self._message_callbacks:
                self._client.message_callback_add(topic, callback)
                self._client.subscribe(topic, 0)

    @except_log
    def _on_disconnect(self, client, userdata, rc):
        self._connected_flag = False
        txt = "Disconnect returned result: {0}".format(mqtt.error_string(rc))
        self.logger.error(txt if rc != 0 else txt + " [Unexpected disconnection]")

    def _process_json_payload(self, msg):
        try:
            payload = json.loads(msg.payload.decode("utf-8"))
        except:
            self.logger.error(traceback.format_exc())
            return None
        return payload

    def _publish(self, topic, payload=None, qos=2, retain=True):
        if payload is None:
            self.logger.error("Payload cannot be None")
            return False
        try:
            publish_info = self._client.publish(topic, payload, qos, retain)
            return publish_info.rc == mqtt.MQTT_ERR_SUCCESS
        except:
            self.logger.error(traceback.format_exc())
            return False

    def _publish_as_json(self, topic, payload=None, qos=0, retain=True):
        try:
            json_payload = json.dumps(payload, separators=(',', ':'), sort_keys=True)
            return self._publish(topic, json_payload, qos, retain)
        except Exception as e:
            self.logger.error(f"Error converting payload to JSON: {str(e)}")
            return False

    def publish(self, message, qos=2, retain=True):
        if isinstance(message, str):
            return self._publish(S.RESULT_MQTT_TOPIC, message, qos, retain)
        elif isinstance(message, dict) or isinstance(message, list):
            return self._publish_as_json(S.RESULT_MQTT_TOPIC, message, qos, retain)
        else:
            self.logger.error("Unsupported message format")
            return False
