#!/usr/bin/env python3

import subprocess
import re
import json
import os
import sys
import time

current_dir = os.path.dirname(os.path.realpath(__file__))
parent_dir = os.path.dirname(current_dir)
sys.path.append(parent_dir)

from lib.sys_utils import IqLogger
from lib.sys_utils import ShellCommand
from lib.cached_network_status import CachedNetworkStatus
from lib.mqtt_client import MQTTClient

def parseRouteOutput(output):
    lines = output.split('\n')
    headers = lines[1].split()
    routes = []

    for line in lines[2:]:
        if line:
            values = line.split()
            routeInfo = dict(zip(headers, values))
            routes.append(routeInfo)

    return routes

def getRoutingTable():
    result = subprocess.run(['route', '-n'], capture_output=True, text=True)
    output = result.stdout
    return parseRouteOutput(output)

def parseNmcliLine(line):
    elements = re.split(r'(?<!\\):', line)
    for i in range(len(elements)):
        elements[i] = elements[i].replace(r'\:', ':')
    return elements

def getActiveConnections():
    result = subprocess.check_output(['nmcli', '-t', '-f', 'NAME,DEVICE,TYPE', 'connection', 'show', '--active'], text=True)
    networks = {}
    for line in result.strip().split('\n'):
        if line.strip():
            elements = parseNmcliLine(line)

            if elements[2] == "gsm":
                elements[1] = "ppp0"

            networks[elements[1]] = elements[0]
    return networks

def getConnectionInfo():
    routingTable = getRoutingTable()
    activeConnections = getActiveConnections()

    connectionInfo = []
    for route in routingTable:
        if route['Iface'] in activeConnections:
            info = {
                'Name': activeConnections[route['Iface']],
                'Interface': route['Iface'],
                'Metric': int(route['Metric']),
                'Flags': route['Flags']
            }
            connectionInfo.append(info)

    return connectionInfo

def connection_info_task(interval):
    while True:
        cached_data = CachedNetworkStatus()
        connection_info = getConnectionInfo()
        with cached_data.lock:
            cached_data.data["Status"] = connection_info
            cached_data.update_time()
            MQTTClient(logger=IqLogger).publish(json.dumps(cached_data.data))
        time.sleep(interval)

if __name__ == "__main__":
    connectionInfo = getConnectionInfo()
    print(json.dumps(connectionInfo, indent=4))
