#!/usr/bin/env python3

import sys
import os
import re
import subprocess
import json
import time

current_dir = os.path.dirname(os.path.realpath(__file__))
parent_dir = os.path.dirname(current_dir)
sys.path.append(parent_dir)

from lib.nmhelper.nmsetup import NMSetupWrapper
from lib.cached_network_status import CachedNetworkStatus
from lib.sys_utils import IqLogger
from lib.mqtt_client import MQTTClient

def parse_nmcli_line(line):
    elements = re.split(r'(?<!\\):', line)
    for i in range(len(elements)):
        elements[i] = elements[i].replace(r'\:', ':')
    return elements

def update_available_wifi_networks():
    try:
        keys = ["active", "ssid", "bssid", "channel", "rate", "signal", "security"]
        result = subprocess.check_output(["nmcli", "-t", "-f", "ACTIVE,SSID,BSSID,CHAN,RATE,SIGNAL,SECURITY", "dev", "wifi"], text=True)
        network_dict = {}

        for line in result.strip().split('\n'):
            if not line.strip():
                continue

            elements = parse_nmcli_line(line)
            if len(elements) != len(keys):
                print(f"Некорректный формат строки: {line}")
                continue

            network = dict(zip(keys, elements))

            if network["active"].lower() == "yes":
                network["active"] = True
            else:
                del network["active"]

            network["channel"] = int(network["channel"])
            network["signal"] = int(network["signal"])

            # Пропускаем сети с пустым ssid или с низким сигналом и неактивные
            if not network["ssid"] or (network["signal"] < 40 and not network.get("active", False)):
                continue

            network_id = (network["ssid"], network["bssid"])
            existing_network = network_dict.get(network_id)

            # Обновляем информацию о сети, если она активна или имеет более сильный сигнал
            if not existing_network or network.get("active", False) or existing_network["signal"] < network["signal"]:
                network_dict[network_id] = network

        networks = list(network_dict.values())

        sorted_networks = sorted(networks, key=lambda x: (-x.get("active", False), -x["signal"]))

        cached_data = CachedNetworkStatus()
        with cached_data.lock:
            cached_data.data["Connections"]["WiFi"] = sorted_networks
            # MQTTClient(logger=IqLogger()).publish(cached_data.data)

        return sorted_networks
    except subprocess.CalledProcessError as e:
        print("Ошибка при выполнении команды nmcli: ", e)
        return []

def manage_wifi_connection(settings):
    wlan_settings = settings.get('wlan0')
    if wlan_settings:
        is_active = wlan_settings.get('is_active')
        con_name = "LegacyWiFi"
        nmsetup = NMSetupWrapper()

        if is_active:
            password = wlan_settings.get('key')
            ssid = wlan_settings.get('ssid')
            priority = 'high'
            nmsetup.upsert_wifi_connection(priority, con_name, ssid, password)
        else:
            nmsetup.remove_connection(con_name)

def wifi_scan_task(interval):
    while True:
        wifi_networks = update_available_wifi_networks()
        time.sleep(interval)

if __name__ == "__main__":
    wifi_networks = update_available_wifi_networks()
    for network in wifi_networks:
        print(network)
