#!/usr/bin/env python3

import json
import os
import subprocess
import sys
import threading
import time
import re
import shlex

from lib.nmhelper.nmsetup import NMSetupWrapper
from lib.sys_utils import IqLogger
from lib.cm_monitor import start_monitoring
from lib.wifi_manager import wifi_scan_task
from lib.network_info import connection_info_task
from lib.network_monitor import network_monitor_task
from lib.modem_manager import modem_info_task, collect_initial_sim_info, execute_command
from lib.mqtt_client import MQTTClient
from lib.cached_network_status import CachedNetworkStatus


def get_last_sim_with_rating(ratings):
    slot_ratings = []

    for slot, imsi_data in ratings.items():
        if not imsi_data:
            continue

        # Найти последнюю IMSI по last_timestamp
        last_imsi = max(imsi_data.items(), key=lambda x: x[1]["last_timestamp"]["timestamp"])
        imsi, data = last_imsi

        # Добавить слот с рейтингом и IMSI
        slot_ratings.append({
            "slot": slot,
            "imsi": imsi,
            "avg_signal_rating": data["avg_signal_rating"]
        })

    # Сортировать слоты по рейтингу в порядке убывания
    slot_ratings.sort(key=lambda x: x["avg_signal_rating"], reverse=True)
    return [item["slot"] for item in slot_ratings]

def execute_command_with_timeout(command, timeout):
    try:
        # Запуск команды через shell с пайплайном
        process = subprocess.Popen(
            command,
            shell=True,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            text=True
        )
        # Ожидание завершения или таймаута
        stdout, stderr = process.communicate(timeout=timeout)
        return_code = process.returncode

        # Обработка ошибок выполнения команды
        if return_code != 0:
            raise subprocess.CalledProcessError(return_code, command, stderr)

        return stdout.strip()

    except subprocess.TimeoutExpired:
        process.kill()
        raise TimeoutError(f"Command timed out after {timeout} seconds.")
    except subprocess.CalledProcessError as e:
        raise RuntimeError(f"Command failed with error: {e.stderr.strip()}") from e

def parse_json_output(output):
    try:
        return json.loads(output)
    except json.JSONDecodeError as e:
        raise ValueError(f"Failed to parse JSON: {e.msg}") from e

def check_and_import(modules):
    missing_modules = []
    for module_name in modules:
        try:
            globals()[module_name] = __import__(module_name)
        except ImportError:
            missing_modules.append(module_name)

    if missing_modules:
        for missing_module in missing_modules:
            print(f"Ошибка: Модуль '{missing_module}' не найден. Установите его с помощью 'pip install {missing_module}'.")
        sys.exit(1)

required_modules = ["paho.mqtt.client", "pyinotify"]
check_and_import(required_modules)

def set_sim_priorities(sorted_slots):
    nmsetup = NMSetupWrapper()
    if not sorted_slots:
        nmsetup.update_priority("low", "wb-gsm-sim1")
        nmsetup.update_priority("low", "wb-gsm-sim2")
    else:
        first_slot = sorted_slots[0]

        if first_slot == "wb-gsm-sim1":
            second_slot = "wb-gsm-sim2"
        else:
            second_slot = "wb-gsm-sim1"

        nmsetup.update_priority("medium", first_slot)
        nmsetup.update_priority("low", second_slot)


if __name__ == "__main__":
    # Финт на случай перезапуска сервиса в процессе работы контроллера
    # Подрузамевается, что SIM карты не будут менять на горячую,
    # значит не нужно перечитывать IMSI SIM карт.
    # После получения IMSI SIM карт, сохраняем в /tmp/ директорию.
    cache_restored = CachedNetworkStatus().restore()
    mqtt_client = MQTTClient(logger=IqLogger())

    wifi_scan_thread = threading.Thread(target=wifi_scan_task, args=(300, ))
    wifi_scan_thread.daemon = True
    wifi_scan_thread.start()

    wifi_scan_thread = threading.Thread(target=connection_info_task, args=(120, ))
    wifi_scan_thread.daemon = True
    wifi_scan_thread.start()

    file_monitor_thread = threading.Thread(target=start_monitoring)
    file_monitor_thread.daemon = True
    file_monitor_thread.start()

    # Первичное заполнение информации по SIM только при включении контроллера
    # т.к. кеш храним в /tmp/
    if not cache_restored:
        time.sleep(10) # ждем обработки настроек сети
        collect_initial_sim_info()

        # Получаем рейтинг сим карт
        current_dir = os.path.dirname(os.path.abspath(__file__))
        check_sims_path = os.path.join(current_dir, "lib/check_sims.py")
        command = f"journalctl -b 0 | {check_sims_path} --rating-json"
        timeout_seconds = 60

        try:
            output = execute_command_with_timeout(command, timeout_seconds)
            print("Command output:", output)
            parsed_data = parse_json_output(output)

            cached_data = CachedNetworkStatus()
            with cached_data.lock:
                cached_data.data["Ratings"] = parsed_data

            cached_data.save()

            sorted_slots = get_last_sim_with_rating(parsed_data)
            print("Slots sorted by signal rating:", sorted_slots)
            set_sim_priorities(sorted_slots)
            execute_command("service wb-connection-manager restart", 60)

        except TimeoutError as te:
            print("Error:", te)
        except RuntimeError as re:
            print("Error:", re)
        except ValueError as ve:
            print("Error:", ve)


    modem_info_thread = threading.Thread(target=modem_info_task, args=(30, ))
    modem_info_thread.daemon = False
    modem_info_thread.start()

    network_monitor_thread = threading.Thread(target=network_monitor_task)
    network_monitor_thread.daemon = False
    network_monitor_thread.start()
